@@ -1,8 +1,8 @@
 use proc_macro2::{Span, TokenStream};
 use syn::spanned::Spanned;
-use syn::{Field as SynField, Ident, Index, Type};
+use syn::{Expr, Field as SynField, Ident, Index, Result, Type};
 
-use attrs::{parse_attributes, AttributeSpanWrapper, FieldAttr, SqlIdentifier};
+use crate::attrs::{parse_attributes, AttributeSpanWrapper, FieldAttr, SqlIdentifier};
 
 pub struct Field {
     pub ty: Type,
@@ -10,15 +10,18 @@ pub struct Field {
     pub name: FieldName,
     column_name: Option<AttributeSpanWrapper<SqlIdentifier>>,
     pub sql_type: Option<AttributeSpanWrapper<Type>>,
+    pub treat_none_as_default_value: Option<AttributeSpanWrapper<bool>>,
+    pub treat_none_as_null: Option<AttributeSpanWrapper<bool>>,
     pub serialize_as: Option<AttributeSpanWrapper<Type>>,
     pub deserialize_as: Option<AttributeSpanWrapper<Type>>,
-    pub select_expression: Option<AttributeSpanWrapper<SelectExpr>>,
+    pub select_expression: Option<AttributeSpanWrapper<Expr>>,
     pub select_expression_type: Option<AttributeSpanWrapper<Type>>,
     pub embed: Option<AttributeSpanWrapper<bool>>,
+    pub skip_insertion: Option<AttributeSpanWrapper<bool>>,
 }
 
 impl Field {
-    pub fn from_struct_field(field: &SynField, index: usize) -> Self {
+    pub fn from_struct_field(field: &SynField, index: usize) -> Result<Self> {
         let SynField {
             ident, attrs, ty, ..
         } = field;
@@ -28,10 +31,13 @@ impl Field {
         let mut serialize_as = None;
         let mut deserialize_as = None;
         let mut embed = None;
+        let mut skip_insertion = None;
         let mut select_expression = None;
         let mut select_expression_type = None;
+        let mut treat_none_as_default_value = None;
+        let mut treat_none_as_null = None;
 
-        for attr in parse_attributes(attrs) {
+        for attr in parse_attributes(attrs)? {
             let attribute_span = attr.attribute_span;
             let ident_span = attr.ident_span;
             match attr.item {
@@ -49,6 +55,20 @@ impl Field {
                         ident_span,
                     })
                 }
+                FieldAttr::TreatNoneAsDefaultValue(_, value) => {
+                    treat_none_as_default_value = Some(AttributeSpanWrapper {
+                        item: value.value,
+                        attribute_span,
+                        ident_span,
+                    })
+                }
+                FieldAttr::TreatNoneAsNull(_, value) => {
+                    treat_none_as_null = Some(AttributeSpanWrapper {
+                        item: value.value,
+                        attribute_span,
+                        ident_span,
+                    })
+                }
                 FieldAttr::SerializeAs(_, value) => {
                     serialize_as = Some(AttributeSpanWrapper {
                         item: Type::Path(value),
@@ -84,6 +104,13 @@ impl Field {
                         ident_span,
                     })
                 }
+                FieldAttr::SkipInsertion(_) => {
+                    skip_insertion = Some(AttributeSpanWrapper {
+                        item: true,
+                        attribute_span,
+                        ident_span,
+                    })
+                }
             }
         }
 
@@ -97,33 +124,36 @@ impl Field {
             FieldName::Unnamed(_) => ty.span(),
         };
 
-        Self {
+        Ok(Self {
             ty: ty.clone(),
             span,
             name,
             column_name,
             sql_type,
+            treat_none_as_default_value,
+            treat_none_as_null,
             serialize_as,
             deserialize_as,
             select_expression,
             select_expression_type,
             embed,
-        }
+            skip_insertion,
+        })
     }
 
-    pub fn column_name(&self) -> SqlIdentifier {
-        self.column_name
-            .as_ref()
-            .map(|a| a.item.clone())
-            .unwrap_or_else(|| match self.name {
-                FieldName::Named(ref x) => x.into(),
-                FieldName::Unnamed(ref x) => {
-                    abort!(
-                    x,
-                    "All fields of tuple structs must be annotated with `#[diesel(column_name)]`"
-                );
-                }
-            })
+    pub fn column_name(&self) -> Result<SqlIdentifier> {
+        let identifier = self.column_name.as_ref().map(|a| a.item.clone());
+        if let Some(identifier) = identifier {
+            Ok(identifier)
+        } else {
+            match self.name {
+                FieldName::Named(ref x) => Ok(x.into()),
+                FieldName::Unnamed(ref x) => Err(syn::Error::new(
+                    x.span(),
+                    "All fields of tuple structs must be annotated with `#[diesel(column_name)]`",
+                )),
+            }
+        }
     }
 
     pub fn ty_for_deserialize(&self) -> &Type {
@@ -137,6 +167,13 @@ impl Field {
     pub(crate) fn embed(&self) -> bool {
         self.embed.as_ref().map(|a| a.item).unwrap_or(false)
     }
+
+    pub(crate) fn skip_insertion(&self) -> bool {
+        self.skip_insertion
+            .as_ref()
+            .map(|a| a.item)
+            .unwrap_or(false)
+    }
 }
 
 pub enum FieldName {
@@ -152,45 +189,3 @@ impl quote::ToTokens for FieldName {
         }
     }
 }
-
-/// We use this instead of directly `syn::Expr` to reduce compilation time
-///
-/// `syn::Expr` does not properly support tuples when `syn/full` feature is
-/// not enabled, and that feature slightly increases compilation time
-#[allow(clippy::large_enum_variant)]
-pub enum SelectExpr {
-    Expr(syn::Expr),
-    Tuple {
-        paren_token: syn::token::Paren,
-        content: proc_macro2::TokenStream,
-    },
-}
-
-impl quote::ToTokens for SelectExpr {
-    fn to_tokens(&self, tokens: &mut TokenStream) {
-        match self {
-            SelectExpr::Expr(ref e) => e.to_tokens(tokens),
-            SelectExpr::Tuple {
-                ref paren_token,
-                ref content,
-            } => paren_token.surround(tokens, |tokens| content.to_tokens(tokens)),
-        }
-    }
-}
-
-impl syn::parse::Parse for SelectExpr {
-    fn parse(input: syn::parse::ParseStream) -> syn::Result<Self> {
-        let lookahead = input.lookahead1();
-
-        if lookahead.peek(syn::token::Paren) {
-            let content;
-            let paren_token = syn::parenthesized!(content in input);
-            Ok(Self::Tuple {
-                paren_token,
-                content: content.parse()?,
-            })
-        } else {
-            input.parse::<syn::Expr>().map(Self::Expr)
-        }
-    }
-}
