@@ -1,19 +1,21 @@
 use proc_macro2::Span;
+use std::slice::from_ref;
 use syn::punctuated::Punctuated;
 use syn::token::Comma;
+use syn::Result;
 use syn::{
     Data, DataStruct, DeriveInput, Field as SynField, Fields, FieldsNamed, FieldsUnnamed, Ident,
     LitBool, Path, Type,
 };
 
-use attrs::{parse_attributes, StructAttr};
-use field::Field;
-use parsers::{BelongsTo, MysqlType, PostgresType, SqliteType};
-use util::camel_to_snake;
+use crate::attrs::{parse_attributes, StructAttr};
+use crate::field::Field;
+use crate::parsers::{BelongsTo, MysqlType, PostgresType, SqliteType};
+use crate::util::camel_to_snake;
 
 pub struct Model {
     name: Path,
-    table_name: Option<Path>,
+    table_names: Vec<Path>,
     pub primary_key_names: Vec<Ident>,
     treat_none_as_default_value: Option<LitBool>,
     treat_none_as_null: Option<LitBool>,
@@ -25,11 +27,16 @@ pub struct Model {
     pub mysql_type: Option<MysqlType>,
     pub sqlite_type: Option<SqliteType>,
     pub postgres_type: Option<PostgresType>,
+    pub check_for_backend: Option<syn::punctuated::Punctuated<syn::TypePath, syn::Token![,]>>,
     fields: Vec<Field>,
 }
 
 impl Model {
-    pub fn from_item(item: &DeriveInput, allow_unit_structs: bool) -> Self {
+    pub fn from_item(
+        item: &DeriveInput,
+        allow_unit_structs: bool,
+        allow_multiple_table: bool,
+    ) -> Result<Self> {
         let DeriveInput {
             data, ident, attrs, ..
         } = item;
@@ -44,12 +51,15 @@ impl Model {
                 ..
             }) => Some(unnamed),
             _ if !allow_unit_structs => {
-                abort_call_site!("This derive can only be used on non-unit structs")
+                return Err(syn::Error::new(
+                    proc_macro2::Span::call_site(),
+                    "This derive can only be used on non-unit structs",
+                ));
             }
             _ => None,
         };
 
-        let mut table_name = None;
+        let mut table_names = vec![];
         let mut primary_key_names = vec![Ident::new("id", Span::call_site())];
         let mut treat_none_as_default_value = None;
         let mut treat_none_as_null = None;
@@ -61,11 +71,21 @@ impl Model {
         let mut mysql_type = None;
         let mut sqlite_type = None;
         let mut postgres_type = None;
+        let mut check_for_backend = None;
 
-        for attr in parse_attributes(attrs) {
+        for attr in parse_attributes(attrs)? {
             match attr.item {
                 StructAttr::SqlType(_, value) => sql_types.push(Type::Path(value)),
-                StructAttr::TableName(_, value) => table_name = Some(value),
+                StructAttr::TableName(ident, value) => {
+                    if !allow_multiple_table && !table_names.is_empty() {
+                        return Err(syn::Error::new(
+                            ident.span(),
+                            "expected a single table name attribute\n\
+                             note: remove this attribute",
+                        ));
+                    }
+                    table_names.push(value)
+                }
                 StructAttr::PrimaryKey(_, keys) => {
                     primary_key_names = keys.into_iter().collect();
                 }
@@ -80,14 +100,17 @@ impl Model {
                 StructAttr::MysqlType(_, val) => mysql_type = Some(val),
                 StructAttr::SqliteType(_, val) => sqlite_type = Some(val),
                 StructAttr::PostgresType(_, val) => postgres_type = Some(val),
+                StructAttr::CheckForBackend(_, b) => {
+                    check_for_backend = Some(b);
+                }
             }
         }
 
         let name = Ident::new(&infer_table_name(&ident.to_string()), ident.span()).into();
 
-        Self {
+        Ok(Self {
             name,
-            table_name,
+            table_names,
             primary_key_names,
             treat_none_as_default_value,
             treat_none_as_null,
@@ -99,27 +122,36 @@ impl Model {
             mysql_type,
             sqlite_type,
             postgres_type,
-            fields: fields_from_item_data(fields),
-        }
+            fields: fields_from_item_data(fields)?,
+            check_for_backend,
+        })
     }
 
-    pub fn table_name(&self) -> &Path {
-        self.table_name.as_ref().unwrap_or(&self.name)
+    pub fn table_names(&self) -> &[Path] {
+        match self.table_names.len() {
+            0 => from_ref(&self.name),
+            _ => &self.table_names,
+        }
     }
 
     pub fn fields(&self) -> &[Field] {
         &self.fields
     }
 
-    pub fn find_column(&self, column_name: &Ident) -> &Field {
+    pub fn find_column(&self, column_name: &Ident) -> Result<&Field> {
         self.fields()
             .iter()
-            .find(|f| f.column_name() == *column_name)
-            .unwrap_or_else(|| abort!(column_name, "No field with column name {}", column_name))
-    }
-
-    pub fn has_table_name_attribute(&self) -> bool {
-        self.table_name.is_some()
+            .find(|f| {
+                f.column_name()
+                    .map(|c| c == *column_name)
+                    .unwrap_or_default()
+            })
+            .ok_or_else(|| {
+                syn::Error::new(
+                    column_name.span(),
+                    format!("No field with column name {column_name}"),
+                )
+            })
     }
 
     pub fn treat_none_as_default_value(&self) -> bool {
@@ -137,16 +169,16 @@ impl Model {
     }
 }
 
-fn fields_from_item_data(fields: Option<&Punctuated<SynField, Comma>>) -> Vec<Field> {
+fn fields_from_item_data(fields: Option<&Punctuated<SynField, Comma>>) -> Result<Vec<Field>> {
     fields
         .map(|fields| {
             fields
                 .iter()
                 .enumerate()
                 .map(|(i, f)| Field::from_struct_field(f, i))
-                .collect::<Vec<_>>()
+                .collect::<Result<Vec<_>>>()
         })
-        .unwrap_or_default()
+        .unwrap_or_else(|| Ok(Vec::new()))
 }
 
 pub fn infer_table_name(name: &str) -> String {
