@@ -3,10 +3,10 @@ use syn::punctuated::Punctuated;
 use syn::token::Comma;
 use syn::{Ident, LitStr};
 
-use util::{parse_eq, unknown_attribute, MYSQL_TYPE_NOTE};
+use crate::util::{parse_eq, unknown_attribute, MYSQL_TYPE_NOTE};
 
 enum Attr {
-    Name(Ident, LitStr),
+    Name(LitStr),
 }
 
 impl Parse for Attr {
@@ -15,9 +15,9 @@ impl Parse for Attr {
         let name_str = name.to_string();
 
         match &*name_str {
-            "name" => Ok(Attr::Name(name, parse_eq(input, MYSQL_TYPE_NOTE)?)),
+            "name" => Ok(Attr::Name(parse_eq(input, MYSQL_TYPE_NOTE)?)),
 
-            _ => unknown_attribute(&name, &["name"]),
+            _ => Err(unknown_attribute(&name, &["name"])),
         }
     }
 }
@@ -32,17 +32,21 @@ impl Parse for MysqlType {
 
         for attr in Punctuated::<Attr, Comma>::parse_terminated(input)? {
             match attr {
-                Attr::Name(_, value) => name = Some(value),
+                Attr::Name(value) => name = Some(value),
             }
         }
 
         if let Some(name) = name {
             Ok(MysqlType { name })
         } else {
-            abort!(
-                input.span(), "expected attribute `name`";
-                help = "The correct format looks like #[diesel({})]", MYSQL_TYPE_NOTE
-            );
+            Err(syn::Error::new(
+                input.span(),
+                format!(
+                    "expected attribute `name`\n\
+                     help: The correct format looks like #[diesel({})]",
+                    MYSQL_TYPE_NOTE
+                ),
+            ))
         }
     }
 }
