@@ -1,12 +1,13 @@
 use proc_macro2::{Span, TokenStream};
-use syn::{DeriveInput, Ident, Index};
+use quote::quote;
+use syn::{parse_quote, DeriveInput, Ident, Index, Result};
 
-use field::Field;
-use model::Model;
-use util::wrap_in_dummy_mod;
+use crate::field::Field;
+use crate::model::Model;
+use crate::util::wrap_in_dummy_mod;
 
-pub fn derive(item: DeriveInput) -> TokenStream {
-    let model = Model::from_item(&item, false);
+pub fn derive(item: DeriveInput) -> Result<TokenStream> {
+    let model = Model::from_item(&item, false, true)?;
 
     let struct_name = &item.ident;
     let field_ty = &model
@@ -21,7 +22,7 @@ pub fn derive(item: DeriveInput) -> TokenStream {
     });
     let sql_type = &(0..model.fields().len())
         .map(|i| {
-            let i = Ident::new(&format!("__ST{}", i), Span::call_site());
+            let i = Ident::new(&format!("__ST{i}"), Span::call_site());
             quote!(#i)
         })
         .collect::<Vec<_>>();
@@ -32,7 +33,7 @@ pub fn derive(item: DeriveInput) -> TokenStream {
         .params
         .push(parse_quote!(__DB: diesel::backend::Backend));
     for id in 0..model.fields().len() {
-        let ident = Ident::new(&format!("__ST{}", id), Span::call_site());
+        let ident = Ident::new(&format!("__ST{id}"), Span::call_site());
         generics.params.push(parse_quote!(#ident));
     }
     {
@@ -43,9 +44,9 @@ pub fn derive(item: DeriveInput) -> TokenStream {
     }
     let (impl_generics, _, where_clause) = generics.split_for_impl();
 
-    wrap_in_dummy_mod(quote! {
+    Ok(wrap_in_dummy_mod(quote! {
         use diesel::deserialize::{self, FromStaticSqlRow, Queryable};
-        use diesel::row::{Row, Field};
+        use diesel::row::{Row as _, Field as _};
         use std::convert::TryInto;
 
         impl #impl_generics Queryable<(#(#sql_type,)*), __DB> for #struct_name #ty_generics
@@ -59,5 +60,5 @@ pub fn derive(item: DeriveInput) -> TokenStream {
                 })
             }
         }
-    })
+    }))
 }
