@@ -1,12 +1,14 @@
 use proc_macro2::TokenStream;
+use quote::quote;
 use quote::ToTokens;
 use syn::parse::{Parse, ParseStream, Result};
 use syn::punctuated::Punctuated;
 use syn::{
-    Attribute, GenericArgument, Generics, Ident, Lit, Meta, MetaNameValue, PathArguments, Type,
+    parenthesized, parse_quote, Attribute, GenericArgument, Generics, Ident, Meta, MetaNameValue,
+    PathArguments, Token, Type,
 };
 
-pub(crate) fn expand(input: SqlFunctionDecl) -> TokenStream {
+pub(crate) fn expand(input: SqlFunctionDecl, legacy_helper_type_and_module: bool) -> TokenStream {
     let SqlFunctionDecl {
         mut attributes,
         fn_token,
@@ -18,15 +20,16 @@ pub(crate) fn expand(input: SqlFunctionDecl) -> TokenStream {
 
     let sql_name = attributes
         .iter()
-        .find(|attr| {
-            attr.parse_meta()
-                .map(|m| m.path().is_ident("sql_name"))
-                .unwrap_or(false)
-        })
+        .find(|attr| attr.meta.path().is_ident("sql_name"))
         .and_then(|attr| {
-            if let Ok(Meta::NameValue(MetaNameValue {
-                lit: Lit::Str(lit), ..
-            })) = attr.parse_meta()
+            if let Meta::NameValue(MetaNameValue {
+                value:
+                    syn::Expr::Lit(syn::ExprLit {
+                        lit: syn::Lit::Str(ref lit),
+                        ..
+                    }),
+                ..
+            }) = attr.meta
             {
                 Some(lit.value())
             } else {
@@ -35,16 +38,12 @@ pub(crate) fn expand(input: SqlFunctionDecl) -> TokenStream {
         })
         .unwrap_or_else(|| fn_name.to_string());
 
-    let is_aggregate = attributes.iter().any(|attr| {
-        attr.parse_meta()
-            .map(|m| m.path().is_ident("aggregate"))
-            .unwrap_or(false)
-    });
+    let is_aggregate = attributes
+        .iter()
+        .any(|attr| attr.meta.path().is_ident("aggregate"));
 
     attributes.retain(|attr| {
-        attr.parse_meta()
-            .map(|m| !m.path().is_ident("sql_name") && !m.path().is_ident("aggregate"))
-            .unwrap_or(true)
+        !attr.meta.path().is_ident("sql_name") && !attr.meta.path().is_ident("aggregate")
     });
 
     let args = &args;
@@ -95,6 +94,8 @@ pub(crate) fn expand(input: SqlFunctionDecl) -> TokenStream {
         numeric_derive = Some(quote!(#[derive(diesel::sql_types::DieselNumericOps)]));
     }
 
+    let helper_type_doc = format!("The return type of [`{fn_name}()`](super::fn_name)");
+
     let args_iter = args.iter();
     let mut tokens = quote! {
         use diesel::{self, QueryResult};
@@ -110,6 +111,7 @@ pub(crate) fn expand(input: SqlFunctionDecl) -> TokenStream {
             #(pub(in super) #type_args: ::std::marker::PhantomData<#type_args>,)*
         }
 
+        #[doc = #helper_type_doc]
         pub type HelperType #ty_generics = #fn_name <
             #(#type_args,)*
             #(<#arg_name as AsExpression<#arg_type>>::Expression,)*
@@ -222,7 +224,7 @@ pub(crate) fn expand(input: SqlFunctionDecl) -> TokenStream {
                         }
                     };
                 }
-                x if x == 1 => {
+                1 => {
                     let arg_name = arg_name[0];
                     let arg_type = arg_type[0];
 
@@ -392,23 +394,44 @@ pub(crate) fn expand(input: SqlFunctionDecl) -> TokenStream {
 
     let args_iter = args.iter();
 
+    let (outside_of_module_helper_type, return_type_path, internals_module_name) =
+        if legacy_helper_type_and_module {
+            (None, quote! { #fn_name::HelperType }, fn_name.clone())
+        } else {
+            let internals_module_name = Ident::new(&format!("{fn_name}_utils"), fn_name.span());
+            (
+                Some(quote! {
+                    #[allow(non_camel_case_types, non_snake_case)]
+                    #[doc = #helper_type_doc]
+                    pub type #fn_name #ty_generics = #internals_module_name::#fn_name <
+                        #(#type_args,)*
+                        #(<#arg_name as ::diesel::expression::AsExpression<#arg_type>>::Expression,)*
+                    >;
+                }),
+                quote! { #fn_name },
+                internals_module_name,
+            )
+        };
+
     quote! {
         #(#attributes)*
         #[allow(non_camel_case_types)]
         pub #fn_token #fn_name #impl_generics (#(#args_iter,)*)
-            -> #fn_name::HelperType #ty_generics
+            -> #return_type_path #ty_generics
         #where_clause
             #(#arg_name: ::diesel::expression::AsExpression<#arg_type>,)*
         {
-            #fn_name::#fn_name {
+            #internals_module_name::#fn_name {
                 #(#arg_struct_assign,)*
                 #(#type_args: ::std::marker::PhantomData,)*
             }
         }
 
+        #outside_of_module_helper_type
+
         #[doc(hidden)]
         #[allow(non_camel_case_types, non_snake_case, unused_imports)]
-        pub(crate) mod #fn_name {
+        pub(crate) mod #internals_module_name {
             #tokens
         }
     }
@@ -431,7 +454,7 @@ impl Parse for SqlFunctionDecl {
         let generics = Generics::parse(input)?;
         let args;
         let _paren = parenthesized!(args in input);
-        let args = args.parse_terminated::<_, Token![,]>(StrictFnArg::parse)?;
+        let args = args.parse_terminated(StrictFnArg::parse, Token![,])?;
         let return_type = if Option::<Token![->]>::parse(input)?.is_some() {
             Type::parse(input)?
         } else {
